package latte;

import java.awt.*;
import java.io.*;
import java.net.URI;
import java.util.ArrayList;
import javax.script.*;
import javax.swing.*;
import javax.swing.text.BadLocationException;

/*
 *
 * @author Andrzej Okuniewski
 * andrzej.okuniewski@gmail.com
 * Copyright (c) 2011 by Andrzej Okuniewski.
 *
 * Program distributed under GNU General Public License
 * see http://www.gnu.org/
 *
 */
public class Main extends javax.swing.JFrame {

    public Main() {

        //ustaw LaF
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            System.out.println("Error setting LAF: " + e);
        }        
        initComponents();
        
        //wyśrodkuj okno
        setLocationRelativeTo(null);
        
        //ustaw ikony w różnych rozmiarach
        ArrayList<Image> iconImages = new ArrayList<Image>();
        int[] sizes = {256, 128, 96, 64, 48, 32, 24, 16, 14, 10, 8};
        for (int i = 0; i < sizes.length; i++) {
            iconImages.add(new ImageIcon(this.getClass().getResource("/latte/latte_ico_" + sizes[i] + ".png")).getImage());
        }
        this.setIconImages(iconImages);

        //ustaw tekst powitalny w konsoli
        out("----------------------------------");
        out("       Welcome to LattE 1.2b      ");
        out("----------------------------------");
        out("         (c) 2011 by A. Okuniewski");
        out("");
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        panel = new javax.swing.JTabbedPane();
        panel_gaussian = new javax.swing.JPanel();
        input_cif_e = new javax.swing.JLabel();
        input_cif = new javax.swing.JTextField();
        input_cif_but = new javax.swing.JButton();
        radius_e = new javax.swing.JLabel();
        radius = new javax.swing.JSlider();
        dft_mol_e = new javax.swing.JLabel();
        dft_mol_c = new javax.swing.JScrollPane();
        dft_mol = new javax.swing.JTextPane();
        mp2_mol_e = new javax.swing.JLabel();
        mp2_mol_c = new javax.swing.JScrollPane();
        mp2_mol = new javax.swing.JTextPane();
        dft_000_e = new javax.swing.JLabel();
        dft_000_c = new javax.swing.JScrollPane();
        dft_000 = new javax.swing.JTextPane();
        mp2_000_e = new javax.swing.JLabel();
        mp2_000_c = new javax.swing.JScrollPane();
        mp2_000 = new javax.swing.JTextPane();
        dft_cry_e = new javax.swing.JLabel();
        dft_cry_c = new javax.swing.JScrollPane();
        dft_cry = new javax.swing.JTextPane();
        out_format = new javax.swing.JComboBox();
        save_but = new javax.swing.JButton();
        panel_calculate = new javax.swing.JPanel();
        calculate_e = new javax.swing.JLabel();
        in_format = new javax.swing.JComboBox();
        autocomplete = new javax.swing.JCheckBox();
        out_dft_mol_e = new javax.swing.JLabel();
        out_dft_mol = new javax.swing.JTextField();
        out_dft_mol_but = new javax.swing.JButton();
        out_mp2_mol_e = new javax.swing.JLabel();
        out_mp2_mol = new javax.swing.JTextField();
        out_mp2_mol_but = new javax.swing.JButton();
        out_dft_cry_e = new javax.swing.JLabel();
        out_dft_cry = new javax.swing.JTextField();
        out_dft_cry_but = new javax.swing.JButton();
        cutoff_e = new javax.swing.JLabel();
        cutoff = new javax.swing.JSlider();
        out_dft_000_e = new javax.swing.JLabel();
        out_dft_000_c = new javax.swing.JScrollPane();
        out_dft_000 = new javax.swing.JList();
        out_dft_000_but = new javax.swing.JButton();
        out_mp2_000_e = new javax.swing.JLabel();
        out_mp2_000_c = new javax.swing.JScrollPane();
        out_mp2_000 = new javax.swing.JList();
        out_mp2_000_but = new javax.swing.JButton();
        calculate_but = new javax.swing.JButton();
        console_c = new javax.swing.JScrollPane();
        console = new javax.swing.JTextPane();
        menu = new javax.swing.JMenuBar();
        topmenu_file = new javax.swing.JMenu();
        menu_quit = new javax.swing.JMenuItem();
        topmenu_help = new javax.swing.JMenu();
        menu_help = new javax.swing.JMenuItem();
        menu_update = new javax.swing.JMenuItem();
        menu_about = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("LattE 1.3β");
        setMinimumSize(new java.awt.Dimension(800, 400));

        panel.setFont(new java.awt.Font("SansSerif", 0, 11));

        input_cif_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        input_cif_e.setText("Specify path to input CIF file:");

        input_cif.setFont(new java.awt.Font("SansSerif", 0, 11));

        input_cif_but.setFont(new java.awt.Font("SansSerif", 0, 11));
        input_cif_but.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Search_16x16.png"))); // NOI18N
        input_cif_but.setText("Browse");
        input_cif_but.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                input_cif_butActionPerformed(evt);
            }
        });

        radius_e.setFont(new java.awt.Font("SansSerif", 0, 11)); // NOI18N
        radius_e.setText("Search for molecules in 6 Å radius...");

        radius.setMaximum(20);
        radius.setToolTipText("");
        radius.setValue(6);
        radius.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                radiusStateChanged(evt);
            }
        });

        dft_mol_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        dft_mol_e.setText("Single molecule DFT:");

        dft_mol.setFont(new java.awt.Font("Monospaced", 0, 11)); // NOI18N
        dft_mol.setText("%nproc=8\n%mem = 128MW\n# B3LYP/6-31G(d,p)\n# maxdisk = 16GB");
        dft_mol_c.setViewportView(dft_mol);

        mp2_mol_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        mp2_mol_e.setText("Single molecule MP2:");

        mp2_mol.setFont(new java.awt.Font("Monospaced", 0, 11)); // NOI18N
        mp2_mol.setText("%nproc=8\n%mem = 128MW\n# MP2/6-31G(d,p)\n# maxdisk = 16GB");
        mp2_mol_c.setViewportView(mp2_mol);

        dft_000_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        dft_000_e.setText("DFT pairs:");

        dft_000.setFont(new java.awt.Font("Monospaced", 0, 11));
        dft_000.setText("%nproc=8\n%mem = 512MW\n# B3LYP/6-31G(d,p)\n# maxdisk = 32GB");
        dft_000_c.setViewportView(dft_000);

        mp2_000_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        mp2_000_e.setText("MP2 pairs:");

        mp2_000.setFont(new java.awt.Font("Monospaced", 0, 11));
        mp2_000.setText("%nproc=8\n%mem = 512MW\n# MP2/6-31G(d,p)\n# maxdisk = 32GB");
        mp2_000_c.setViewportView(mp2_000);

        dft_cry_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        dft_cry_e.setText("Periodic DFT:");

        dft_cry.setFont(new java.awt.Font("Monospaced", 0, 11));
        dft_cry.setText("%nproc=8\n%mem = 4GW\n# B3LYP/3-21G*\n# maxdisk = 128GB");
        dft_cry_c.setViewportView(dft_cry);

        out_format.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_format.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "GJF", "COM" }));
        out_format.setToolTipText("Type of input Gaussian files");
        out_format.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                out_formatItemStateChanged(evt);
            }
        });

        save_but.setFont(new java.awt.Font("SansSerif", 0, 11));
        save_but.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Save_16x16.png"))); // NOI18N
        save_but.setText("Save GJF files...");
        save_but.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                save_butActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout panel_gaussianLayout = new javax.swing.GroupLayout(panel_gaussian);
        panel_gaussian.setLayout(panel_gaussianLayout);
        panel_gaussianLayout.setHorizontalGroup(
            panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panel_gaussianLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(input_cif_e)
                    .addGroup(panel_gaussianLayout.createSequentialGroup()
                        .addComponent(input_cif, javax.swing.GroupLayout.DEFAULT_SIZE, 608, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(input_cif_but))
                    .addComponent(radius_e)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panel_gaussianLayout.createSequentialGroup()
                        .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(dft_mol_e)
                            .addComponent(dft_mol_c, javax.swing.GroupLayout.DEFAULT_SIZE, 135, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(mp2_mol_e)
                            .addComponent(mp2_mol_c, javax.swing.GroupLayout.DEFAULT_SIZE, 137, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(panel_gaussianLayout.createSequentialGroup()
                                .addComponent(dft_000_c, javax.swing.GroupLayout.DEFAULT_SIZE, 135, Short.MAX_VALUE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED))
                            .addGroup(panel_gaussianLayout.createSequentialGroup()
                                .addComponent(dft_000_e)
                                .addGap(92, 92, 92)))
                        .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(mp2_000_e)
                            .addComponent(mp2_000_c, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 135, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(dft_cry_e)
                            .addComponent(dft_cry_c, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 139, Short.MAX_VALUE)))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panel_gaussianLayout.createSequentialGroup()
                        .addComponent(save_but, javax.swing.GroupLayout.DEFAULT_SIZE, 650, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(out_format, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(radius, javax.swing.GroupLayout.DEFAULT_SIZE, 705, Short.MAX_VALUE))
                .addContainerGap())
        );
        panel_gaussianLayout.setVerticalGroup(
            panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panel_gaussianLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(input_cif_e)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(input_cif, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(input_cif_but))
                .addGap(18, 18, 18)
                .addComponent(radius_e)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(radius, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(dft_mol_e)
                    .addComponent(mp2_mol_e)
                    .addComponent(dft_cry_e)
                    .addComponent(dft_000_e)
                    .addComponent(mp2_000_e))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(dft_000_c, javax.swing.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE)
                    .addComponent(mp2_mol_c, javax.swing.GroupLayout.Alignment.TRAILING, 0, 158, Short.MAX_VALUE)
                    .addComponent(dft_mol_c, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE)
                    .addComponent(mp2_000_c, javax.swing.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE)
                    .addComponent(dft_cry_c, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panel_gaussianLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(save_but)
                    .addComponent(out_format, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        panel.addTab("Prepare Gaussian files", new javax.swing.ImageIcon(getClass().getResource("/latte/gaussian.png")), panel_gaussian); // NOI18N

        calculate_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        calculate_e.setText("Specify path to Gaussian's OUT output files...");

        in_format.setFont(new java.awt.Font("SansSerif", 0, 11));
        in_format.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "OUT", "LOG" }));
        in_format.setToolTipText("Type of output Gaussian files");
        in_format.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                in_formatItemStateChanged(evt);
            }
        });

        autocomplete.setFont(new java.awt.Font("SansSerif", 0, 11));
        autocomplete.setSelected(true);
        autocomplete.setText("Autocompletion");

        out_dft_mol_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_dft_mol_e.setText("Single molecule DFT:");

        out_dft_mol.setFont(new java.awt.Font("SansSerif", 0, 11));

        out_dft_mol_but.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_dft_mol_but.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Search_16x16.png"))); // NOI18N
        out_dft_mol_but.setText("Browse");
        out_dft_mol_but.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                out_dft_mol_butActionPerformed(evt);
            }
        });

        out_mp2_mol_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_mp2_mol_e.setText("Single molecule MP2:");

        out_mp2_mol.setFont(new java.awt.Font("SansSerif", 0, 11));

        out_mp2_mol_but.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_mp2_mol_but.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Search_16x16.png"))); // NOI18N
        out_mp2_mol_but.setText("Browse");
        out_mp2_mol_but.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                out_mp2_mol_butActionPerformed(evt);
            }
        });

        out_dft_cry_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_dft_cry_e.setText("Periodic DFT:");

        out_dft_cry.setFont(new java.awt.Font("SansSerif", 0, 11));

        out_dft_cry_but.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_dft_cry_but.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Search_16x16.png"))); // NOI18N
        out_dft_cry_but.setText("Browse");
        out_dft_cry_but.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                out_dft_cry_butActionPerformed(evt);
            }
        });

        cutoff_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        cutoff_e.setText("Use 0 Å cutoff (0 = no cutoff)...");

        cutoff.setFont(new java.awt.Font("SansSerif", 0, 11));
        cutoff.setMaximum(20);
        cutoff.setValue(0);
        cutoff.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                cutoffStateChanged(evt);
            }
        });

        out_dft_000_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_dft_000_e.setText("DFT pairs:");

        out_dft_000.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_dft_000_c.setViewportView(out_dft_000);

        out_dft_000_but.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_dft_000_but.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Search_16x16.png"))); // NOI18N
        out_dft_000_but.setText("Browse");
        out_dft_000_but.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                out_dft_000_butActionPerformed(evt);
            }
        });

        out_mp2_000_e.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_mp2_000_e.setText("MP2 pairs:");

        out_mp2_000.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_mp2_000_c.setViewportView(out_mp2_000);

        out_mp2_000_but.setFont(new java.awt.Font("SansSerif", 0, 11));
        out_mp2_000_but.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Search_16x16.png"))); // NOI18N
        out_mp2_000_but.setText("Browse");
        out_mp2_000_but.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                out_mp2_000_butActionPerformed(evt);
            }
        });

        calculate_but.setFont(new java.awt.Font("SansSerif", 0, 11));
        calculate_but.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/compute.png"))); // NOI18N
        calculate_but.setText("Calculate lattice energy");
        calculate_but.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                calculate_butActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout panel_calculateLayout = new javax.swing.GroupLayout(panel_calculate);
        panel_calculate.setLayout(panel_calculateLayout);
        panel_calculateLayout.setHorizontalGroup(
            panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panel_calculateLayout.createSequentialGroup()
                .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(panel_calculateLayout.createSequentialGroup()
                        .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(panel_calculateLayout.createSequentialGroup()
                                .addGap(19, 19, 19)
                                .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(panel_calculateLayout.createSequentialGroup()
                                        .addComponent(out_dft_cry, javax.swing.GroupLayout.DEFAULT_SIZE, 280, Short.MAX_VALUE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(out_dft_cry_but, javax.swing.GroupLayout.DEFAULT_SIZE, 92, Short.MAX_VALUE))
                                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panel_calculateLayout.createSequentialGroup()
                                        .addComponent(out_mp2_mol, javax.swing.GroupLayout.DEFAULT_SIZE, 280, Short.MAX_VALUE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(out_mp2_mol_but))
                                    .addComponent(out_dft_cry_e)))
                            .addGroup(panel_calculateLayout.createSequentialGroup()
                                .addGap(20, 20, 20)
                                .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(out_dft_mol_e)
                                    .addGroup(panel_calculateLayout.createSequentialGroup()
                                        .addComponent(in_format, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(autocomplete))
                                    .addGroup(panel_calculateLayout.createSequentialGroup()
                                        .addComponent(out_dft_mol, javax.swing.GroupLayout.DEFAULT_SIZE, 279, Short.MAX_VALUE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(out_dft_mol_but, javax.swing.GroupLayout.DEFAULT_SIZE, 92, Short.MAX_VALUE))
                                    .addComponent(out_mp2_mol_e)))
                            .addGroup(panel_calculateLayout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(calculate_e)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(cutoff_e)
                            .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(cutoff, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 312, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panel_calculateLayout.createSequentialGroup()
                                    .addComponent(out_dft_000_c, javax.swing.GroupLayout.DEFAULT_SIZE, 149, Short.MAX_VALUE)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(out_mp2_000_c, javax.swing.GroupLayout.DEFAULT_SIZE, 157, Short.MAX_VALUE)))
                            .addGroup(panel_calculateLayout.createSequentialGroup()
                                .addComponent(out_dft_000_e)
                                .addGap(101, 101, 101)
                                .addComponent(out_mp2_000_e))))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panel_calculateLayout.createSequentialGroup()
                        .addGap(403, 403, 403)
                        .addComponent(out_dft_000_but, javax.swing.GroupLayout.DEFAULT_SIZE, 149, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(out_mp2_000_but, javax.swing.GroupLayout.DEFAULT_SIZE, 157, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panel_calculateLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(calculate_but, javax.swing.GroupLayout.DEFAULT_SIZE, 705, Short.MAX_VALUE)))
                .addContainerGap())
        );

        panel_calculateLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {out_dft_cry_but, out_dft_mol_but, out_mp2_mol_but});

        panel_calculateLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {out_dft_000_but, out_dft_000_c});

        panel_calculateLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {out_mp2_000_but, out_mp2_000_c});

        panel_calculateLayout.setVerticalGroup(
            panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panel_calculateLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(calculate_e)
                    .addComponent(cutoff_e))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(in_format, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(autocomplete))
                    .addComponent(cutoff, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(out_dft_mol_e)
                    .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(out_dft_000_e)
                        .addComponent(out_mp2_000_e)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(out_mp2_000_c, javax.swing.GroupLayout.DEFAULT_SIZE, 191, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, panel_calculateLayout.createSequentialGroup()
                        .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(out_dft_mol, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(out_dft_mol_but))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(out_mp2_mol_e, javax.swing.GroupLayout.PREFERRED_SIZE, 14, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(out_mp2_mol, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(out_mp2_mol_but))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(out_dft_cry_e)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(out_dft_cry, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(out_dft_cry_but)))
                    .addComponent(out_dft_000_c, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 191, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panel_calculateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(out_dft_000_but)
                    .addComponent(out_mp2_000_but))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(calculate_but)
                .addContainerGap())
        );

        panel.addTab("Calculate lattice energy", new javax.swing.ImageIcon(getClass().getResource("/latte/compute.png")), panel_calculate); // NOI18N

        console_c.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        console_c.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        console.setEditable(false);
        console.setFont(new java.awt.Font("Monospaced", 0, 11));
        console_c.setViewportView(console);

        menu.setFont(new java.awt.Font("SansSerif", 0, 11));

        topmenu_file.setText("File");
        topmenu_file.setFont(new java.awt.Font("SansSerif", 0, 11));

        menu_quit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.event.InputEvent.CTRL_MASK));
        menu_quit.setFont(new java.awt.Font("SansSerif", 0, 11));
        menu_quit.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Remove_16x16.png"))); // NOI18N
        menu_quit.setText("Quit");
        menu_quit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menu_quitActionPerformed(evt);
            }
        });
        topmenu_file.add(menu_quit);

        menu.add(topmenu_file);

        topmenu_help.setText("Help");
        topmenu_help.setFont(new java.awt.Font("SansSerif", 0, 11));

        menu_help.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0));
        menu_help.setFont(new java.awt.Font("SansSerif", 0, 11));
        menu_help.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Help_16x16.png"))); // NOI18N
        menu_help.setText("Help");
        menu_help.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menu_helpActionPerformed(evt);
            }
        });
        topmenu_help.add(menu_help);

        menu_update.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F2, 0));
        menu_update.setFont(new java.awt.Font("SansSerif", 0, 11));
        menu_update.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Synchronize_16x16.png"))); // NOI18N
        menu_update.setText("Update");
        menu_update.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menu_updateActionPerformed(evt);
            }
        });
        topmenu_help.add(menu_update);

        menu_about.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F3, 0));
        menu_about.setFont(new java.awt.Font("SansSerif", 0, 11));
        menu_about.setIcon(new javax.swing.ImageIcon(getClass().getResource("/latte/Information_16x16.png"))); // NOI18N
        menu_about.setText("About...");
        menu_about.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menu_aboutActionPerformed(evt);
            }
        });
        topmenu_help.add(menu_about);

        menu.add(topmenu_help);

        setJMenuBar(menu);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(panel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 730, Short.MAX_VALUE)
                    .addComponent(console_c, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 730, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(panel, javax.swing.GroupLayout.PREFERRED_SIZE, 376, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(console_c, javax.swing.GroupLayout.DEFAULT_SIZE, 106, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void menu_quitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menu_quitActionPerformed
        System.exit(0);
    }//GEN-LAST:event_menu_quitActionPerformed

    private void menu_helpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menu_helpActionPerformed
        try {
            Desktop.getDesktop().browse(new URI("http://www.kchn.pg.gda.pl/latte/?p=manual"));
        } catch (Exception e) {
        }
    }//GEN-LAST:event_menu_helpActionPerformed

    private void menu_updateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menu_updateActionPerformed
        try {
            Desktop.getDesktop().browse(new URI("http://www.kchn.pg.gda.pl/latte/?p=download&ver=1.3b"));
        } catch (Exception e) {
        }
    }//GEN-LAST:event_menu_updateActionPerformed

    private void menu_aboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menu_aboutActionPerformed
        // Wyświetl okno informacyjne
        JFrame info = new latte.Info();
        info.setVisible(true);
        latte.Main.getWindows()[0].setEnabled(false);
    }//GEN-LAST:event_menu_aboutActionPerformed

    private void input_cif_butActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_input_cif_butActionPerformed
        openFile(input_cif, ".cif", false);
    }//GEN-LAST:event_input_cif_butActionPerformed

    private void radiusStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_radiusStateChanged
        // zmiana etykiety podczas zmiany pozycji suwaka
        radius_e.setText("Search for molecules in " + radius.getValue() + " Å radius...");
    }//GEN-LAST:event_radiusStateChanged

    private void out_formatItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_out_formatItemStateChanged
        //zmiana nazwy pola zapisywania
        save_but.setText("Save " + out_format.getSelectedItem().toString() + " files...");
    }//GEN-LAST:event_out_formatItemStateChanged

    private void save_butActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_save_butActionPerformed
        try {

            //ścieżka do pliku CIF
            File file = new File(input_cif.getText());

            //policz ile linii ma plik
            BufferedReader inputfile = new BufferedReader(new FileReader(file));
            int i = 0, j = 0;
            while ((inputfile.readLine()) != null) {
                i++;
            }
            inputfile.close();

            //przepisz plik do tablicy
            String[] cif = new String[i];
            String str;
            inputfile = new BufferedReader(new FileReader(file));
            i = 0;
            while ((str = inputfile.readLine()) != null) {
                cif[i] = str;
                i++;
            }
            inputfile.close();

            //deklaracja zmiennych
            String grupa = "?";
            Float cell_a = 0f, cell_b = 0f, cell_c = 0f, cell_alpha = 0f, cell_beta = 0f, cell_gamma = 0f;
            String[][] opsym = null;
            int pos_label = 0, pos_x = 0, pos_y = 0, pos_z = 0;
            String[] symbol = null;
            Float[][] coord = null;
            Float[][][] mole = null;

            for (i = 0; i < cif.length; i++) {
                //pobieranie podstawowych informacji
                if (cif[i].contains("_symmetry_space_group_name_H-M")) {
                    grupa = strim(cif[i]);
                }

                //pobieranie parametrów komórki elementarnej
                if (cif[i].contains("_cell_length_a")) {
                    cell_a = ftrim(cif[i].substring(cif[i].indexOf(" ")).trim());
                }
                if (cif[i].contains("_cell_length_b")) {
                    cell_b = ftrim(cif[i].substring(cif[i].indexOf(" ")).trim());
                }
                if (cif[i].contains("_cell_length_c")) {
                    cell_c = ftrim(cif[i].substring(cif[i].indexOf(" ")).trim());
                }
                if (cif[i].contains("_cell_angle_alpha")) {
                    cell_alpha = ftrim(cif[i].substring(cif[i].indexOf(" ")).trim());
                }
                if (cif[i].contains("_cell_angle_beta")) {
                    cell_beta = ftrim(cif[i].substring(cif[i].indexOf(" ")).trim());
                }
                if (cif[i].contains("_cell_angle_gamma")) {
                    cell_gamma = ftrim(cif[i].substring(cif[i].indexOf(" ")).trim());
                }

                //pobieranie operacji symetrii
                if (cif[i].contains("_symmetry_equiv_pos_as_xyz")) {
                    //sprawdzanie ilości operacji symetrii
                    j = 1;
                    while (!cif[i + j].contains("_") && !cif[i + j].trim().equals("")) {
                        j++;
                    }

                    opsym = new String[j - 1][4];

                    j = 1;
                    while (!cif[i + j].contains("_") && !cif[i + j].trim().equals("")) {
                        opsym[j - 1][1] = cif[i + j].replace("'", "").replace(" ", "").split(",")[0];
                        opsym[j - 1][2] = cif[i + j].replace("'", "").replace(" ", "").split(",")[1];
                        opsym[j - 1][3] = cif[i + j].replace("'", "").replace(" ", "").split(",")[2];
                        j++;
                    }
                }

                //pobieranie listy atomów
                if (cif[i].contains("_atom_site_label") && !cif[i].contains("geom")) {
                    while (!cif[i].contains("loop_")) {
                        i--;
                    }

                    //wyszukiwanie pozycji etykiet i współrzędnych
                    j = 0;
                    while (cif[i].contains("_")) {
                        if (cif[i].contains("_atom_site_label")) {
                            pos_label = j - 1;
                        }
                        if (cif[i].contains("_atom_site_fract_x")) {
                            pos_x = j - 1;
                        }
                        if (cif[i].contains("_atom_site_fract_y")) {
                            pos_y = j - 1;
                        }
                        if (cif[i].contains("_atom_site_fract_z")) {
                            pos_z = j - 1;
                        }
                        i++;
                        j++;
                    }

                    //obliczanie ilości atomów
                    j = 0;
                    while (!cif[i + j].contains("_") && !cif[i + j].trim().equals("")) {
                        j++;
                    }

                    symbol = new String[j];
                    coord = new Float[j][4];

                    //wczytywanie atomów do tablic
                    j = 0;
                    while (!cif[i + j].contains("_") && !cif[i + j].trim().equals("")) {
                        symbol[j] = stripLabel(cif[i + j].trim().replace("  ", " ").split(" ")[pos_label]);
                        coord[j][1] = ftrim(cif[i + j].trim().replace("  ", " ").split(" ")[pos_x]);
                        coord[j][2] = ftrim(cif[i + j].trim().replace("  ", " ").split(" ")[pos_y]);
                        coord[j][3] = ftrim(cif[i + j].trim().replace("  ", " ").split(" ")[pos_z]);
                        j++;
                    }
                }
            }

            out("");
            out("LattE has found crystal with " + grupa + " space group symmetry.");
            out("");
            out("Cell parameters:");
            out("a = " + cell_a + ", b = " + cell_b + ", c = " + cell_c);
            out("\\a = " + cell_alpha + ", \\b = " + cell_beta + ", \\g = " + cell_gamma);
            out("");
            out("Symmetry operations:");
            for (i = 0; i < opsym.length; i++) {
                out(i + 1 + ") " + opsym[i][1] + ", " + opsym[i][2] + ", " + opsym[i][3]);
            }

            out("");
            out("In assymmetric part there is " + coord.length + " atoms:");
            for (i = 0; i < coord.length; i++) {
                out(i + 1 + ") " + symbol[i] + " " + fieldRound(coord[i][1]) + " " + fieldRound(coord[i][2]) + " " + fieldRound(coord[i][3]));
            }

            out("");
            out("In unit cell there is " + opsym.length + " molecules, " + coord.length + " atoms each.");

            //powołaj macierz komórki elementarnej
            mole = new Float[opsym.length][coord.length][4];

            ScriptEngineManager manager = new ScriptEngineManager();
            ScriptEngine engine = manager.getEngineByName("js");

            for (i = 0; i < opsym.length; i++) {
                for (j = 0; j < coord.length; j++) {
                    //wykonaj operacje symetrii
                    try {
                        mole[i][j][1] = Float.valueOf(engine.eval(opsym[i][1].replace("x", " " + coord[j][1].toString() + " ").replace("y", " " + coord[j][2].toString() + " ").replace("z", " " + coord[j][3].toString() + " ")).toString());
                        mole[i][j][2] = Float.valueOf(engine.eval(opsym[i][2].replace("x", " " + coord[j][1].toString() + " ").replace("y", " " + coord[j][2].toString() + " ").replace("z", " " + coord[j][3].toString() + " ")).toString());
                        mole[i][j][3] = Float.valueOf(engine.eval(opsym[i][3].replace("x", " " + coord[j][1].toString() + " ").replace("y", " " + coord[j][2].toString() + " ").replace("z", " " + coord[j][3].toString() + " ")).toString());
                        //out((i + 1) + "." + (j + 1) + ") " + symbol[j] + " " + fieldRound(mole[i][j][1]) + " " + fieldRound(mole[i][j][2]) + " " + fieldRound(mole[i][j][3]));
                    } catch (ScriptException e) {
                    }
                }
            }

            //oblicz współczynniki transformacji
            Float xx = 0f, xy = 0f, xz = 0f, yy = 0f, yz = 0f, zz = 0f;
            xx = cell_a;
            xy = (float) (cell_b * Math.cos(Math.PI * cell_gamma / 180));
            xz = (float) (cell_c * Math.cos(Math.PI * cell_beta / 180));
            yy = (float) (cell_b * Math.sin(Math.PI * cell_gamma / 180));
            yz = (float) (cell_c * (Math.cos(Math.PI * cell_alpha / 180) - Math.cos(Math.PI * cell_beta / 180) * Math.cos(Math.PI * cell_gamma / 180)) / (Math.sin(Math.PI * cell_gamma / 180)));
            zz = (float) ((cell_c * Math.sqrt(1 - Math.pow(Math.cos(Math.PI * cell_alpha / 180), 2) - Math.pow(Math.cos(Math.PI * cell_beta / 180), 2) - Math.pow(Math.cos(Math.PI * cell_gamma / 180), 2) + 2 * Math.cos(Math.PI * cell_alpha / 180) * Math.cos(Math.PI * cell_beta / 180) * Math.cos(Math.PI * cell_gamma / 180))) / (Math.sin(Math.PI * cell_gamma / 180)));

            Integer R = radius.getValue();
            Integer N = 0, id = 0;
            Boolean doNext = true;
            Float R2 = (float) Math.pow(R, 2);
            Float Rmin = 0f, Rmin2 = 0f, d2 = 0f;


            // oblicz wsp. ortogonalne dla cząsteczki centralnej
            Float[][] mole0 = new Float[coord.length][4];
            for (i = 0; i < mole0.length; i++) {
                mole0[i][1] = (float) mole[0][i][1] * xx + mole[0][i][2] * xy + mole[0][i][3] * xz;
                mole0[i][2] = (float) mole[0][i][2] * yy + mole[0][i][3] * yz;
                mole0[i][3] = (float) mole[0][i][3] * zz;
            }

            Float[][] moleN = new Float[coord.length][4];

            String name = file.getParent() + File.separator + file.getName().substring(0, file.getName().lastIndexOf(".")) + "_";
            String ext = "." + out_format.getSelectedItem().toString().toLowerCase();

            String eol = "\r\n";
            if (out_format.getSelectedItem().toString().equals("COM")) {
                eol = "\n";
            }

            String preambleDFT = "";
            String preambleMP2 = "";

            File fDFT = null;
            FileWriter fwDFT = null;

            File fMP2 = null;
            FileWriter fwMP2 = null;


            //wygeneruj i zapisz pojedyncze cząsteczki MP2 i DFT
            preambleMP2 = "";
            for (i = 0; i < mp2_mol.getDocument().getDefaultRootElement().getElementCount(); i++) {
                try {
                    preambleMP2 += mp2_mol.getDocument().getText(mp2_mol.getDocument().getDefaultRootElement().getElement(i).getStartOffset(), mp2_mol.getDocument().getDefaultRootElement().getElement(i).getEndOffset() - mp2_mol.getDocument().getDefaultRootElement().getElement(i).getStartOffset()).trim() + eol;
                } catch (BadLocationException e) {
                }
            }

            preambleDFT = "";
            for (i = 0; i < dft_mol.getDocument().getDefaultRootElement().getElementCount(); i++) {
                try {
                    preambleDFT += dft_mol.getDocument().getText(dft_mol.getDocument().getDefaultRootElement().getElement(i).getStartOffset(), dft_mol.getDocument().getDefaultRootElement().getElement(i).getEndOffset() - dft_mol.getDocument().getDefaultRootElement().getElement(i).getStartOffset()).trim() + eol;
                } catch (BadLocationException e) {
                }
            }


            fDFT = new File(name + "dft_mol" + ext);
            fwDFT = new FileWriter(fDFT);
            fwDFT.write(preambleDFT + eol + "Single molecule DFT calculation." + eol + eol + "0 1" + eol);

            fMP2 = new File(name + "mp2_mol" + ext);
            fwMP2 = new FileWriter(fMP2);
            fwMP2.write(preambleDFT + eol + "Single molecule MP2 calculation." + eol + eol + "0 1" + eol);

            for (int p = 0; p < coord.length; p++) { // zmiana atomów cząsteczki
                Float x = (float) mole0[p][1] * xx + mole0[p][2] * xy + mole0[p][3] * xz;
                Float y = (float) mole0[p][2] * yy + mole0[p][3] * yz;
                Float z = (float) mole0[p][3] * zz;
                fwDFT.write(symbol[p] + " " + fieldRound(x) + " " + fieldRound(y) + " " + fieldRound(z) + eol);
                fwMP2.write(symbol[p] + " " + fieldRound(x) + " " + fieldRound(y) + " " + fieldRound(z) + eol);
            }

            fwDFT.write(eol);
            fwMP2.write(eol);
            fwDFT.close();
            fwMP2.close();


            //wygeneruj i zapisz plik periodyczny DFT
            preambleDFT = "";
            for (i = 0; i < dft_cry.getDocument().getDefaultRootElement().getElementCount(); i++) {
                try {
                    preambleDFT += dft_cry.getDocument().getText(dft_cry.getDocument().getDefaultRootElement().getElement(i).getStartOffset(), dft_cry.getDocument().getDefaultRootElement().getElement(i).getEndOffset() - dft_cry.getDocument().getDefaultRootElement().getElement(i).getStartOffset()).trim() + eol;
                } catch (BadLocationException e) {
                }
            }

            fDFT = new File(name + "dft_cry" + ext);
            fwDFT = new FileWriter(fDFT);
            fwDFT.write(preambleDFT + eol + "Periodic DFT calculation." + eol + eol + "0 1" + eol);

            //tu wszystkie atomy w komórce <0,1)
            for (int h = -1; h <= 1; h++) { //zmaina na x
                for (int k = -1; k <= 1; k++) { // zmiana na y
                    for (int l = -1; l <= 1; l++) { // zmiana na z
                        for (i = 0; i < opsym.length; i++) { // wybór cząsteczki w komórce
                            for (int p = 0; p < coord.length; p++) { // zmiana atomów cząsteczki
                                Float x = (float) mole[i][p][1] + h;
                                Float y = (float) mole[i][p][2] + k;
                                Float z = (float) mole[i][p][3] + l;
                                if (x >= 0 && x < 1 && y >= 0 && y < 1 && z >= 0 && z < 1) {
                                    x = (float) x * xx + y * xy + z * xz;
                                    y = (float) y * yy + z * yz;
                                    z = (float) z * zz;
                                    fwDFT.write(symbol[p] + " " + fieldRound(x) + " " + fieldRound(y) + " " + fieldRound(z) + eol);
                                }
                            }
                        }
                    }
                }
            }
            fwDFT.write("Tv " + fieldRound(xx) + " " + fieldRound(0f) + " " + fieldRound(0f) + eol);
            fwDFT.write("Tv " + fieldRound(xy) + " " + fieldRound(yy) + " " + fieldRound(0f) + eol);
            fwDFT.write("Tv " + fieldRound(xz) + " " + fieldRound(yz) + " " + fieldRound(zz) + eol);
            fwDFT.write(eol);
            fwDFT.close();

            //wygeneruj i zapisz pary MP2 i DFT
            preambleMP2 = "";
            for (i = 0; i < mp2_000.getDocument().getDefaultRootElement().getElementCount(); i++) {
                try {
                    preambleMP2 += mp2_000.getDocument().getText(mp2_000.getDocument().getDefaultRootElement().getElement(i).getStartOffset(), mp2_000.getDocument().getDefaultRootElement().getElement(i).getEndOffset() - mp2_000.getDocument().getDefaultRootElement().getElement(i).getStartOffset()).trim() + eol;
                } catch (BadLocationException e) {
                }
            }

            preambleDFT = "";
            for (i = 0; i < dft_000.getDocument().getDefaultRootElement().getElementCount(); i++) {
                try {
                    preambleDFT += dft_000.getDocument().getText(dft_000.getDocument().getDefaultRootElement().getElement(i).getStartOffset(), dft_000.getDocument().getDefaultRootElement().getElement(i).getEndOffset() - dft_000.getDocument().getDefaultRootElement().getElement(i).getStartOffset()).trim() + eol;
                } catch (BadLocationException e) {
                }
            }

            out("");
            out("Searching for molecules within " + R + " A radius in surrounding unit cells:");

            //pętala porównywania odległości i zapisu do plików MP2
            while (doNext) {
                doNext = false;
                out("N = " + N);
                for (int h = -N; h <= N; h++) { //zmaina na x
                    for (int k = -N; k <= N; k++) { // zmiana na y
                        for (int l = -N; l <= N; l++) { // zmiana na z

                            if (Math.abs(h) == N || Math.abs(k) == N || Math.abs(l) == N) {

                                for (i = 0; i < opsym.length; i++) { // wybór cząsteczki do pary

                                    Rmin2 = 1000f;
                                    for (int p = 0; p < coord.length; p++) { // zmiana atomów cząsteczki centralnej
                                        for (int q = 0; q < coord.length; q++) { // zmiana atomów cząsteczki do pary
                                            moleN[q][1] = (float) (mole[i][q][1] + h) * xx + (mole[i][q][2] + k) * xy + (mole[i][q][3] + l) * xz;
                                            moleN[q][2] = (float) (mole[i][q][2] + k) * yy + (mole[i][q][3] + l) * yz;
                                            moleN[q][3] = (float) (mole[i][q][3] + l) * zz;

                                            d2 = (float) (Math.pow(mole0[p][1] - moleN[q][1], 2) + Math.pow(mole0[p][2] - moleN[q][2], 2) + Math.pow(mole0[p][3] - moleN[q][3], 2));

                                            if (d2 < Rmin2) {
                                                Rmin2 = d2;
                                            }
                                        }
                                    }
                                    if (Rmin2 <= R2) {
                                        doNext = true;
                                        Rmin = (float) Math.sqrt(Rmin2);

                                        out(" Molecule no. " + (i + 1) + " in (" + h + "," + k + "," + l + ") unit cell has Rmin = " + Rmin + " A.");

                                        if (id > 0) {
                                            fMP2 = new File(name + "mp2_" + onFields(id) + ext);
                                            fwMP2 = new FileWriter(fMP2);

                                            fDFT = new File(name + "dft_" + onFields(id) + ext);
                                            fwDFT = new FileWriter(fDFT);

                                            fwMP2.write(preambleMP2 + eol + "Pair 0-" + id + " (MP2), Rmin = " + Rmin + " A." + eol + eol + "0 1" + eol);
                                            fwDFT.write(preambleDFT + eol + "Pair 0-" + id + " (DFT), Rmin = " + Rmin + " A." + eol + eol + "0 1" + eol);

                                            for (int p = 0; p < coord.length; p++) { // wypisz cząsteczkę 0 do pliku
                                                fwMP2.write(symbol[p] + " " + fieldRound(mole0[p][1]) + " " + fieldRound(mole0[p][2]) + " " + fieldRound(mole0[p][3]) + eol);
                                                fwDFT.write(symbol[p] + " " + fieldRound(mole0[p][1]) + " " + fieldRound(mole0[p][2]) + " " + fieldRound(mole0[p][3]) + eol);
                                            }
                                            for (int p = 0; p < coord.length; p++) { // wypisz cząsteczkę n do pliku
                                                fwMP2.write(symbol[p] + " " + fieldRound(moleN[p][1]) + " " + fieldRound(moleN[p][2]) + " " + fieldRound(moleN[p][3]) + eol);
                                                fwDFT.write(symbol[p] + " " + fieldRound(moleN[p][1]) + " " + fieldRound(moleN[p][2]) + " " + fieldRound(moleN[p][3]) + eol);
                                            }
                                            fwMP2.write(eol);
                                            fwDFT.write(eol);
                                            fwMP2.close();
                                            fwDFT.close();
                                        }
                                        id++;
                                    }
                                }

                            }

                        }
                    }
                }
                N++;
            }

            out(" No more surrounding molecules found.");


            out("");
            out(out_format.getSelectedItem().toString() + " files for DFT and MP2 calculations were saved in:");
            out(file.getParent());

        } catch (IOException e) {
        }
    }//GEN-LAST:event_save_butActionPerformed

    private void in_formatItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_in_formatItemStateChanged
        //zmiana etykiety
        calculate_e.setText("Specify path to Gaussian's " + in_format.getSelectedItem().toString() + " output files...");
    }//GEN-LAST:event_in_formatItemStateChanged

    private void out_dft_mol_butActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_out_dft_mol_butActionPerformed
        openFile(out_dft_mol, "dft_mol." + in_format.getSelectedItem().toString().toLowerCase(), autocomplete.isSelected());
    }//GEN-LAST:event_out_dft_mol_butActionPerformed

    private void out_mp2_mol_butActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_out_mp2_mol_butActionPerformed
        openFile(out_mp2_mol, "mp2_mol." + in_format.getSelectedItem().toString().toLowerCase(), autocomplete.isSelected());
    }//GEN-LAST:event_out_mp2_mol_butActionPerformed

    private void out_dft_cry_butActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_out_dft_cry_butActionPerformed
        openFile(out_dft_cry, "dft_cry." + in_format.getSelectedItem().toString().toLowerCase(), autocomplete.isSelected());
    }//GEN-LAST:event_out_dft_cry_butActionPerformed

    private void out_dft_000_butActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_out_dft_000_butActionPerformed
        //otwieranie plików par DFT
        try {
            class FileFilter extends javax.swing.filechooser.FileFilter {

                public boolean accept(File f) {
                    return f.isDirectory() || f.getName().toLowerCase().endsWith("dft_001." + in_format.getSelectedItem().toString().toLowerCase());
                }

                public String getDescription() {
                    return "DFT_001." + in_format.getSelectedItem().toString() + " files";
                }
            }

            final JFileChooser fc = new JFileChooser();
            fc.setFileFilter(new FileFilter());
            int returnVal = fc.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();

                String name = file.getParent() + File.separator + file.getName().substring(0, file.getName().lastIndexOf("_")) + "_";
                String ext = "." + in_format.getSelectedItem().toString().toLowerCase();

                DefaultListModel modelDFT = new DefaultListModel();
                out_dft_000.setModel(modelDFT);

                int i = 1;
                while ((file = new File(name + onFields(i) + ext)).exists()) {
                    modelDFT.add(out_dft_000.getModel().getSize(), file);
                    i++;
                }
                out(name + onFields(1) + ".." + onFields(i - 1) + ext + " opened.");
            } else {
                out("Open command cancelled by user.");
            }
        } catch (Exception e) {
            out("Error opening file: " + e);
        }
    }//GEN-LAST:event_out_dft_000_butActionPerformed

    private void out_mp2_000_butActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_out_mp2_000_butActionPerformed
        //otwieranie plików par MP2
        try {
            class FileFilter extends javax.swing.filechooser.FileFilter {

                public boolean accept(File f) {
                    return f.isDirectory() || f.getName().toLowerCase().endsWith("mp2_001." + in_format.getSelectedItem().toString().toLowerCase());
                }

                public String getDescription() {
                    return "MP2_001." + in_format.getSelectedItem().toString() + " files";
                }
            }

            final JFileChooser fc = new JFileChooser();
            fc.setFileFilter(new FileFilter());
            int returnVal = fc.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                String name = file.getParent() + File.separator + file.getName().substring(0, file.getName().lastIndexOf("_")) + "_";
                String ext = "." + in_format.getSelectedItem().toString().toLowerCase();

                DefaultListModel modelMP2 = new DefaultListModel();
                out_mp2_000.setModel(modelMP2);

                int i = 1;
                while ((file = new File(name + onFields(i) + ext)).exists()) {
                    modelMP2.add(out_mp2_000.getModel().getSize(), file);
                    i++;
                }
                out(name + onFields(1) + ".." + onFields(i - 1) + ext + " opened.");
            } else {
                out("Open command cancelled by user.");
            }
        } catch (Exception e) {
            out("Error opening file: " + e);
        }
    }//GEN-LAST:event_out_mp2_000_butActionPerformed

    private void calculate_butActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_calculate_butActionPerformed
        //OBLICZANIE ENERGII SIECI NA PODSTAWIE PLIKÓW GAUSSIANA

        //Sprawdź czy wszystkie pliki podano i istnieją
        boolean error = false;

        if (out_dft_mol.getText().trim().equals("")) {
            error = true;
            out("Error: Single molecule DFT output file not specified.");
        } else if (!new File(out_dft_mol.getText()).exists()) {
            error = true;
            out("Error: Single molecule DFT output file not found.");
        }

        if (out_mp2_mol.getText().trim().equals("")) {
            error = true;
            out("Error: Single molecule MP2 output file not specified.");
        } else if (!new File(out_mp2_mol.getText()).exists()) {
            error = true;
            out("Error: Single molecule MP2 output file not found.");
        }

        if (out_dft_cry.getText().trim().equals("")) {
            error = true;
            out("Error: Periodic DFT output file not specified.");
        } else if (!new File(out_dft_cry.getText()).exists()) {
            error = true;
            out("Error: Periodic DFT output file not found.");
        }

        if (out_dft_000.getModel().getElementAt(0).toString().trim().equals("")) {
            error = true;
            out("Error: DFT pair output files not specified.");
        } else if (!new File(out_dft_000.getModel().getElementAt(0).toString()).exists()) {
            error = true;
            out("Error: DFT pair output files not found.");
        }

        if (out_mp2_000.getModel().getElementAt(0).toString().trim().equals("")) {
            error = true;
            out("Error: MP2 pair output files not specified.");
        } else if (!new File(out_mp2_000.getModel().getElementAt(0).toString()).exists()) {
            error = true;
            out("Error: MP2 pair output files not found.");
        }

        //Jeśli wszystkie pliki określono prawidłowo
        if (!error) {

            File f = null;
            BufferedReader br = null;
            String line = null;
            String temp = null;
            out("");

            try {
                //Odszukaj wartość energii pojedynczej cząsteczki DFT: Edft_mol
                Double Edft_mol = 0d;
                br = new BufferedReader(new FileReader(new File(out_dft_mol.getText())));
                temp = null;
                while ((line = br.readLine()) != null) {
                    if (line.contains("SCF Done:")) {
                        temp = line.substring(line.indexOf("=") + 1).trim();
                        temp = temp.substring(0, temp.indexOf(" ")).trim();
                        Edft_mol = Double.valueOf(temp);
                    }
                }
                br.close();                
                out("Edft_mol = " + Edft_mol + " Ha");

                //Odszukaj wartość energii pojedynczej cząsteczki MP2: Emp2_mol
                Double Emp2_mol = 0d;
                br = new BufferedReader(new FileReader(new File(out_mp2_mol.getText())));
                temp = null;
                while ((line = br.readLine()) != null) {
                    if (line.contains("EUMP2 =")) {
                        temp = line.substring(line.indexOf("EUMP2 =") + 7).trim();
                        temp = temp.toLowerCase().replace("d", "e").trim();
                        Emp2_mol = Double.valueOf(temp);
                    }
                }
                br.close();
                out("Emp2_mol = " + Emp2_mol + " Ha");

                //Odszukaj wartość energii kryształu DFT: Edft_cry
                Double Edft_cry = 0d;
                br = new BufferedReader(new FileReader(new File(out_dft_cry.getText())));
                temp = null;
                while ((line = br.readLine()) != null) {
                    if (line.contains("SCF Done:")) {
                        temp = line.substring(line.indexOf("=") + 1).trim();
                        temp = temp.substring(0, temp.indexOf(" ")).trim();
                        Edft_cry = Double.valueOf(temp);
                    }
                }
                br.close();
                out("Edft_cry = " + Edft_cry + " Ha");

                //Oszacuj wartość Z
                Integer Z = (int) Math.round(Edft_cry / Edft_mol);
                out("");
                out("Z = " + Z);

                //Oblicz Elatt_0 = [1/Z Edft_cry - Edft_mol]
                Double Elatt_0 = (Edft_cry / Z - Edft_mol);
                out("");
                out("Simple lattice energy: ");
                out(" Elatt_0 = " + fieldRound(new Float(Elatt_0)) + " Ha");
                out("         = " + fieldRound(new Float(Elatt_0 * 2625.49962)) + " kJ/mol");
                out("         = " + fieldRound(new Float(Elatt_0 * 627.509469)) + " kcal/mol");
                out("");

                out("Calculation of MP2 corrections...");

                if (out_dft_000.getModel().getSize() == out_mp2_000.getModel().getSize()) {

                    Double Edft_i = 0d, Emp2_i = 0d, Sum = 0d, Rmin = 0d;
                    Double dE = null;

                    for (int i = 0; i < out_dft_000.getModel().getSize(); i++) {
                        br = new BufferedReader(new FileReader(new File(out_dft_000.getModel().getElementAt(i).toString())));
                        temp = null;
                        Rmin = null;
                        while ((line = br.readLine()) != null) {
                            if (line.contains("SCF Done:")) {
                                temp = line.substring(line.indexOf("=") + 1).trim();
                                temp = temp.substring(0, temp.indexOf(" ")).trim();
                                Edft_i = Double.valueOf(temp);
                            }

                            if (line.contains(", Rmin =") && Rmin == null) {
                                temp = line.substring(line.indexOf(", Rmin =") + 8).trim();
                                temp = temp.substring(0, temp.indexOf(" ")).trim();
                                Rmin = Double.valueOf(temp);
                            }
                        }
                        br.close();


                        br = new BufferedReader(new FileReader(new File(out_mp2_000.getModel().getElementAt(i).toString())));
                        temp = null;
                        while ((line = br.readLine()) != null) {
                            if (line.contains("EUMP2 =")) {
                                temp = line.substring(line.indexOf("EUMP2 =") + 7).trim();
                                temp = temp.toLowerCase().replace("d", "e").trim();
                                Emp2_i = Double.valueOf(temp);
                            }
                        }
                        br.close();

                        dE = ((Emp2_i / 2 - Emp2_mol) - (Edft_i / 2 - Edft_mol));

                        if(cutoff.getValue() > 0) {
                            if(Rmin < cutoff.getValue()) {
                                Sum += dE;
                                out("dE_" + (i + 1) + " = " + dE + " Ha, Rmin = " + Rmin + " A");
                            } else {
                                out("dE_" + (i + 1) + " = " + dE + " Ha, Rmin = " + Rmin + " A - rejected");
                            }
                        } else {
                            Sum += dE;
                            out("dE_" + (i + 1) + " = " + dE + " Ha, Rmin = " + Rmin + " A");
                        }
                    }

                    out("SUM(dE) = " + Sum + " Ha");

                    Double Elatt = Elatt_0 + Sum;

                    out("");
                    out("MP2-corrected lattice energy: ");
                    out(" Elatt = " + fieldRound(new Float(Elatt)) + " Ha");
                    out("       = " + fieldRound(new Float(Elatt * 2625.49962)) + " kJ/mol");
                    out("       = " + fieldRound(new Float(Elatt * 627.509469)) + " kcal/mol");
                    out("");

                } else {
                    out("Error: Number of DFT and MP2 pair files is not equal.");
                }

            } catch (IOException e) {
            }
        }
    }//GEN-LAST:event_calculate_butActionPerformed

    private void cutoffStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_cutoffStateChanged
        // zmiana etykiety podczas zmiany pozycji suwaka
        cutoff_e.setText("Use " + cutoff.getValue() + " Å cutoff (0 = no cutoff)...");
    }//GEN-LAST:event_cutoffStateChanged

    public final void out(String output) {
        //dodaj linię tekstu do konsoli
        console.setText(console.getText() + output + "\n");
    }

    public void openFile(JTextField dest, final String format, boolean expand) {
        //otwieranie pliku
        try {
            class FileFilter extends javax.swing.filechooser.FileFilter {

                public boolean accept(File f) {
                    return f.isDirectory() || f.getName().toLowerCase().endsWith(format);
                }

                public String getDescription() {
                    return format.toUpperCase() + " files";
                }
            }

            final JFileChooser fc = new JFileChooser();
            fc.setFileFilter(new FileFilter());

            int returnVal = fc.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                dest.setText(fc.getSelectedFile().getPath());
                out(fc.getSelectedFile().getPath() + " opened.");

                if (expand) {
                    out("");
                    out("Attempting to auto-find remaining files...");
                    String name = "";
                    name = fc.getSelectedFile().getParent() + File.separator + fc.getSelectedFile().getName().substring(0, fc.getSelectedFile().getName().lastIndexOf("."));
                    name = name.substring(0, name.lastIndexOf("_"));
                    name = name.substring(0, name.lastIndexOf("_")) + "_";
                    String ext = "." + in_format.getSelectedItem().toString().toLowerCase();

                    if (new File(name + "dft_mol" + ext).exists()) {
                        out_dft_mol.setText(name + "dft_mol" + ext);
                        out(" File _dft_mol auto-completed.");
                    }

                    if (new File(name + "mp2_mol" + ext).exists()) {
                        out_mp2_mol.setText(name + "mp2_mol" + ext);
                        out(" File _mp2_mol auto-completed.");
                    }

                    if (new File(name + "dft_cry" + ext).exists()) {
                        out_dft_cry.setText(name + "dft_cry" + ext);
                        out(" File _dft_cry auto-completed.");
                    }

                    DefaultListModel modelDFT = new DefaultListModel();
                    DefaultListModel modelMP2 = new DefaultListModel();
                    File file = null;
                    int i = 0;

                    i = 1;
                    out_dft_000.setModel(modelDFT);
                    while ((file = new File(name + "dft_" + onFields(i) + ext)).exists()) {
                        modelDFT.add(out_dft_000.getModel().getSize(), file);
                        i++;
                    }
                    out(" Files _dft_" + onFields(1) + ".." + onFields(i - 1) + " auto-completed.");

                    i = 1;
                    out_mp2_000.setModel(modelMP2);
                    while ((file = new File(name + "mp2_" + onFields(i) + ext)).exists()) {
                        modelMP2.add(out_mp2_000.getModel().getSize(), file);
                        i++;
                    }
                    out(" Files _mp2_" + onFields(1) + ".." + onFields(i - 1) + " auto-completed.");

                }

            } else {
                out("Open command cancelled by user.");
            }

        } catch (Exception e) {
            out("Error opening file: " + e);
        }


    }

    public static Float ftrim(String input) {
        //wczytywanie liczby z linii wraz z usuwaniem niepewności
        if (input.contains("(")) {
            return Float.valueOf(input.substring(0, input.indexOf("(")));
        } else {
            return Float.valueOf(input);
        }
    }

    public static String strim(String input) {
        //wczytywanie ciągu znaków z linni z usunięciem apostrofów
        return input.substring(input.indexOf(" ")).trim().replace("'", "");
    }

    public static String onFields(Integer input) {
        //formatuje liczbę na 3 polach: 000
        if (input < 10) {
            return "00" + input.toString();
        } else if (input < 100) {
            return "0" + input.toString();
        } else {
            return input.toString();
        }
    }

    public static String fieldRound(Float input) {
        //zaokrągla liczbę i formatuje na polach: +0000.0000
        if ((input > 0 && input < 0.001) || (input < 0 && input > -0.001)) {
            return "    0.000";
        } else {
            String znak = " ";
            if (input < 0) {
                znak = "-";
            }

            String cecha = Integer.toString(Math.abs(Integer.valueOf(input.toString().substring(0, input.toString().indexOf(".")))));
            String mantysa = input.toString().substring(input.toString().indexOf(".") + 1);

            if (cecha.length() < 1) {
                cecha = "   " + znak + "0";
            } else if (cecha.length() < 2) {
                cecha = "   " + znak + cecha;
            } else if (cecha.length() < 3) {
                cecha = "  " + znak + cecha;
            } else if (cecha.length() < 4) {
                cecha = " " + znak + cecha;
            } else {
                cecha = znak + cecha;
            }

            if (mantysa.length() < 1) {
                mantysa = "000";
            } else if (mantysa.length() < 2) {
                mantysa = mantysa + "00";
            } else if (mantysa.length() < 3) {
                mantysa = mantysa + "0";
            } else if (mantysa.length() > 3) {
                mantysa = mantysa.substring(0, 3);
            }

            return cecha + "." + mantysa;
        }
    }

    public static String stripLabel(String input) {
        //ekstrakcja symboli pierwiastków
        boolean found = false;
        String char_list = "0123456789";
        int tmp = 0, pos = 0;
        for (int i = 0; i < char_list.length(); i++) {
            tmp = input.indexOf(char_list.charAt(i));
            if (tmp > -1) {
                if (!found) {
                    pos = tmp;
                    found = true;
                    continue;
                }
                pos = Math.min(pos, tmp);
            }
        }
        if (!found) {
            return input;
        } else {
            return input.substring(0, pos);
        }
    }

    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new latte.Main().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox autocomplete;
    private javax.swing.JButton calculate_but;
    private javax.swing.JLabel calculate_e;
    private javax.swing.JTextPane console;
    private javax.swing.JScrollPane console_c;
    private javax.swing.JSlider cutoff;
    private javax.swing.JLabel cutoff_e;
    private javax.swing.JTextPane dft_000;
    private javax.swing.JScrollPane dft_000_c;
    private javax.swing.JLabel dft_000_e;
    private javax.swing.JTextPane dft_cry;
    private javax.swing.JScrollPane dft_cry_c;
    private javax.swing.JLabel dft_cry_e;
    private javax.swing.JTextPane dft_mol;
    private javax.swing.JScrollPane dft_mol_c;
    private javax.swing.JLabel dft_mol_e;
    private javax.swing.JComboBox in_format;
    private javax.swing.JTextField input_cif;
    private javax.swing.JButton input_cif_but;
    private javax.swing.JLabel input_cif_e;
    private javax.swing.JMenuBar menu;
    private javax.swing.JMenuItem menu_about;
    private javax.swing.JMenuItem menu_help;
    private javax.swing.JMenuItem menu_quit;
    private javax.swing.JMenuItem menu_update;
    private javax.swing.JTextPane mp2_000;
    private javax.swing.JScrollPane mp2_000_c;
    private javax.swing.JLabel mp2_000_e;
    private javax.swing.JTextPane mp2_mol;
    private javax.swing.JScrollPane mp2_mol_c;
    private javax.swing.JLabel mp2_mol_e;
    private javax.swing.JList out_dft_000;
    private javax.swing.JButton out_dft_000_but;
    private javax.swing.JScrollPane out_dft_000_c;
    private javax.swing.JLabel out_dft_000_e;
    private javax.swing.JTextField out_dft_cry;
    private javax.swing.JButton out_dft_cry_but;
    private javax.swing.JLabel out_dft_cry_e;
    private javax.swing.JTextField out_dft_mol;
    private javax.swing.JButton out_dft_mol_but;
    private javax.swing.JLabel out_dft_mol_e;
    private javax.swing.JComboBox out_format;
    private javax.swing.JList out_mp2_000;
    private javax.swing.JButton out_mp2_000_but;
    private javax.swing.JScrollPane out_mp2_000_c;
    private javax.swing.JLabel out_mp2_000_e;
    private javax.swing.JTextField out_mp2_mol;
    private javax.swing.JButton out_mp2_mol_but;
    private javax.swing.JLabel out_mp2_mol_e;
    private javax.swing.JTabbedPane panel;
    private javax.swing.JPanel panel_calculate;
    private javax.swing.JPanel panel_gaussian;
    private javax.swing.JSlider radius;
    private javax.swing.JLabel radius_e;
    private javax.swing.JButton save_but;
    private javax.swing.JMenu topmenu_file;
    private javax.swing.JMenu topmenu_help;
    // End of variables declaration//GEN-END:variables
}
